from setuptools import setup

setup(name='pydesy',
      version='0.0.2',
      description='Geodetic library',
      packages=['pydesy'],
      author_email='shmakov.maxim2001@yandex.ru',
      zip_safe=False)
